#define _16bit OSL_PF_8888
#define spr_draw oslDrawImageXY
#define MAXSPRITES 1024
#define ROOM_WIDTH 5
#define MAXACTORS 37

#include <oslib/oslib.h>
#include <oslib/libpspmath/pspmath.h>
#include <psputils.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspmpeg.h>
#include <pspdisplay.h>
#include <psprtc.h>
#include <psppower.h>
#include <pspfpu.h>
#include <pspgu.h>
#include <psptypes.h>
#include "common.h"
#include "room.h"
#include "quicksort.h"
#include "define_room.h"
#include "sprite_struct.h"
#include <stdio.h>
#include <stdlib.h>

extern OSL_COLOR colors[3];

int *initTilesSprite()
{
	static int gT[100];
	int i;

	for (i=0; i<100; i++)
	{
		gT[i]=0;
	}

	return gT;
}

int *initTiles()
{
	static int gT[100];
	int i;

	for (i=0; i<100; i++)
	{
		gT[i]=1;
	}

	return gT;
}

int *initnesw()
{
	static int gT[4];
	int i;

	for (i=0; i<4; i++)
	{
		gT[i]=0;
	}

	return gT;
}

void loadColors(int currentRoom)
{
	char bouffeur[50];
	SceUID checkfile;

	sprintf(bouffeur, "./room/cols%d.qfpc", currentRoom);
	checkfile = sceIoOpen(bouffeur, PSP_O_RDONLY, 0777);

	sceIoRead(checkfile, colors, sizeof(colors));
	sceIoClose(checkfile);
}

void saveRoom(int *groundTiles, int *spriteTiles, int *nesw, int levelno)
{
	int k;
	short bad=0;

	char path_buffer[50];
	SceUID checkfile;

	for (k=0; k<100; k++)
	{
		if (k<10) groundTiles[k]=0;
		if (k>89) groundTiles[k]=0;
		if (!(k%10)) groundTiles[k]=0;
		if ((k%10) == 9) groundTiles[k]=0;
	}

	sprintf(path_buffer, "./room/room%d.qfpm", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_WRONLY|PSP_O_CREAT, 0777);

	k=sceIoWrite(checkfile, groundTiles, sizeof(groundTiles)*100);
	if (!k) bad=1;
	sceIoClose(checkfile);

	sprintf(path_buffer, "./room/sprs%d.qfps", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_WRONLY|PSP_O_CREAT, 0777);

	k=sceIoWrite(checkfile, spriteTiles, sizeof(spriteTiles)*100);
	if (!k) bad=1;
	sceIoClose(checkfile);

	sprintf(path_buffer, "./room/door%d.qfpd", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_WRONLY|PSP_O_CREAT, 0777);

	k=sceIoWrite(checkfile, nesw, sizeof(nesw)*4);
	if (!k) bad=1;
	sceIoClose(checkfile);

	sprintf(path_buffer, "./room/cols%d.qfpc", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_WRONLY|PSP_O_CREAT, 0777);
	k=sceIoWrite(checkfile, colors, sizeof(colors));
	if (!k) bad=1;
	sceIoClose(checkfile);

	if (bad)
	{
		oslStartDrawing();
		oslDrawFillRect(0, 0, 480, 272, RGB(32, 32, 32));

		center(64, "Error while writing data!", 0, RGB(255,0,0));

		oslEndDrawing();
		oslSwapBuffers();
		oslWaitVSync();

		oslWaitKey();
	}
}

int *loadDoors(int levelno)
{
	static int doors[4];

	char path_buffer[50];
	SceUID checkfile;

	sprintf(path_buffer, "./room/door%d.qfpd", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_RDONLY, 0777);

	sceIoRead(checkfile, doors, sizeof(doors));
	sceIoClose(checkfile);

	return doors;
}

int *loadSprites(int levelno)
{
	static int sT[100];

	char path_buffer[50];
	SceUID checkfile;

	sprintf(path_buffer, "./room/sprs%d.qfps", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_RDONLY, 0777);

	if (checkfile < 0)
	{
		oslStartDrawing();
		oslDrawFillRect(0, 0, 480, 272, RGB(0, 0, 255));
		oslEndDrawing();
		oslSwapBuffers();
		oslWaitVSync();
		oslQuit();
	}

	sceIoRead(checkfile, sT, sizeof(sT));
	sceIoClose(checkfile);

	return sT;
}

int *loadRoom(int levelno)
{
	static int gT[100];

	char path_buffer[50];
	SceUID checkfile;

	sprintf(path_buffer, "./room/room%d.qfpm", levelno);
	checkfile = sceIoOpen(path_buffer, PSP_O_RDONLY, 0777);

	if (checkfile < 0)
	{
		oslStartDrawing();
		oslDrawFillRect(0, 0, 480, 272, RGB(255, 0, 0));
		oslEndDrawing();
		oslSwapBuffers();
		oslWaitVSync();
		oslQuit();
	}

	sceIoRead(checkfile, gT, sizeof(gT));
	sceIoClose(checkfile);

	return gT;
}

OSL_COLOR pickColor(OSL_COLOR c)
{
	int d=256;
	short quit=0, comps=2;
	char s[12];

	while (!quit)
	{
		oslStartDrawing();
		oslDrawFillRect(0, 0, 480, 272, RGB(0, 0, 0));
		oslDrawFillRect(0, 0, 480, 136, c);
     	oslDrawFillRect(0, 166+8*comps, 480, 174+8*comps, RGB(255, 0, 0));

		sprintf(s, "%lu", c);
		center(150, s, 1, RGB(255, 255, 0));

		sprintf(s, "B: %lu", (c/65536)%256);
		center(182, s, 1, RGB(255, 255, 0));

		sprintf(s, "G: %lu", (c/256)%256);
		center(174, s, 1, RGB(255, 255, 0));

		sprintf(s, "R: %lu", (c%256));
		center(166, s, 1, RGB(255, 255, 0));

		oslEndDrawing();
		oslSwapBuffers();
		oslWaitVSync();

		oslReadKeys();

		d=vfpu_powf(comps, 256);

		if ((osl_keys-> held.left) && ((c/d)%256 > 0))
		{
			c-=d;
		}

		if ((osl_keys-> held.right) && ((c/d)%256 < 255))
		{
			c+=d;
		}

		if (osl_keys->pressed.down)
		{
			comps=(comps+1)%3;
		}

		if (osl_keys->pressed.up)
		{
			comps--;
			if (comps < 0) comps=2;
		}


		if (osl_keys-> pressed.cross)
		{
			quit=1;
		}
	}

	return c;
}

void lveditor()
{
	int i, *groundTiles, *spriteTiles, *nesw, levelno=12, maxcases=(ROOM_WIDTH * ROOM_WIDTH)-1, maxsprites=MAXACTORS, maxtiles=12;
	short currentTile=1, currentSprite=0, quit=0, cx=0, cy=0, dispMenu=0, ch=0, chosen=0;
	char bouffeur[30], roomno[30];

	OSL_IMAGE *tiles[maxtiles];
	OSL_IMAGE *sprites[maxsprites];

	nesw=initnesw();

	for (i=0; i<maxtiles; i++)
	{
		if (i<10) sprintf(bouffeur, "./lved/grd0%d.png", i); else sprintf(bouffeur, "./lved/grd%d.png", i);
		tiles[i] = oslLoadImageFilePNG(bouffeur, OSL_IN_RAM, OSL_PF_8888);
	}

	for (i=0; i<maxsprites; i++)
	{
		if (i<10) sprintf(bouffeur, "./gfx/actor0%d.png", i); else sprintf(bouffeur, "./gfx/actor%d.png", i);
		sprites[i] = oslLoadImageFilePNG(bouffeur, OSL_IN_RAM, OSL_PF_8888);
		oslSetImageTileSize(sprites[i], 0, 0, 32, sprites[i]->sizeY);
		sprites[i] -> stretchX = 16;
		sprites[i] -> stretchY = 16;
	}

	groundTiles=initTiles();
	spriteTiles=initTilesSprite();

	colors[0]=RGB(32, 32, 32);
	colors[1]=RGB(255, 255, 255);
	colors[2]=RGB(0, 255, 255);

	while (!quit)
	{
		oslStartDrawing();
		oslDrawFillRect(0, 0, 480, 272, RGB(32, 32, 32));

		for (i=0; i<100; i++)
		{
			if ((i>0) && (i<99) && (i != 90) && (i != 9))
			{
				if (((i/10)==0) || ((i/10)==9))
				{
					oslSetAlpha(OSL_FX_ALPHA,64);
				}
				else
				{
					oslSetAlpha(OSL_FX_DEFAULT, 0);
					if (((i%10)==0) || ((i%10)==9))
					{
						oslSetAlpha(OSL_FX_ALPHA,64);
					}
				}

				setFont(0);
				oslSetAlpha(OSL_FX_TINT, colors[1]);
				spr_draw(tiles[groundTiles[i]], 224+(i%10)*16, 80+(i/10)*16);
				oslSetAlpha(OSL_FX_DEFAULT, 0);

				if (spriteTiles[i] > 0)
				{
					oslSetAlpha(OSL_FX_DEFAULT, 0);
					spr_draw(sprites[spriteTiles[i]], 224+(i%10)*16, 80+(i/10)*16);
				}
			}

			if ((i<10) || (i>89) || ((i%10)==0) || ((i%10)==9))
			{
				oslSetAlpha(OSL_FX_DEFAULT, 0);
				oslDrawFillRect(224+(i%10)*16, 80+(i/10)*16, 224+(i%10)*16+16, 80+(i/10)*16+16, colors[0]);
			}

		}

		oslSetAlpha(OSL_FX_DEFAULT, 0);
		setFont(2);

		if (nesw[0])
		{
			oslDrawFillRect(288,80,320,96, colors[2]);
			oslDrawString(224, 0, "N");
		}
		if (nesw[1])
		{
			oslDrawFillRect(368,144,384,176, colors[2]);
			oslDrawString(236, 0, "E");
		}
		if (nesw[2])
		{
			oslDrawFillRect(288,224,320,240, colors[2]);
			oslDrawString(248, 0, "S");
		}
		if (nesw[3])
		{
			oslDrawFillRect(224,144,240,176, colors[2]);
			oslDrawString(260, 0, "W");
		}

		sprintf(bouffeur, "%d", currentTile);
		oslDrawString(4, 28, bouffeur);
		spr_draw(tiles[currentTile], 24, 24);

		sprintf(bouffeur, "%d", currentSprite);
		oslDrawString(4, 52, bouffeur);
		spr_draw(sprites[currentSprite], 24, 56);

		oslDrawRect(224+cx*16, 80+cy*16, 240+cx*16, 96+cy*16, RGB(random(64)+192, random(64)+192, random(64)+192));

		if (dispMenu)
		{
			oslDrawFillRect(0, 0, 480, 272, RGBA(32, 32, 32, 128));
			oslDrawFillRect(0, 76+20*ch, 480, 88+20*ch, RGB(255, 0, 0));

			center(76, "PICK WALL COLOR", 2, RGB(255, 255, 255));
			center(96, "PICK GROUND COLOR", 2, RGB(255, 255, 255));
			center(116, "PICK DOOR COLOR", 2, RGB(255, 255, 255));
			center(136, "CLEAR ROOM", 2, RGB(255, 255, 255));
			center(156, "SAVE ROOM", 2, RGB(255, 255, 255));
			center(176, "LOAD ROOM", 2, RGB(255, 255, 255));
			center(196, "QUIT EDITOR", 2, RGB(255, 255, 255));
		}

		oslEndDrawing();
		oslSwapBuffers();
		oslWaitVSync();

		oslReadKeys();

		if (!dispMenu)
		{
			if (osl_keys->pressed.L)
			{
				if (osl_keys->held.square)
				{
					if (currentSprite > 0) currentSprite--;
				}
				else
				{
					 if (currentTile > 0) currentTile--;
				}
			}

			if (osl_keys->pressed.R)
			{
				if (osl_keys->held.square)
				{
					if (currentSprite < (maxsprites-1)) currentSprite++;
				}
				else
				{
					 if (currentTile < (maxtiles-1)) currentTile++;
				}
			}

			if ((osl_keys->pressed.left) && (cx > 0))
			{
				cx--;
			}

			if ((osl_keys->pressed.right) && (cx < 9))
			{
				cx++;
			}

			if ((osl_keys->pressed.up) && (cy > 0))
			{
				cy--;
			}

			if ((osl_keys->pressed.down) && (cy < 9))
			{
				cy++;
			}

			if ((osl_keys->pressed.cross) && ((cx > 0) && (cx < 9)) && ((cy > 0) && (cy < 9)) )
			{
				if ((spriteTiles[cy*10+cx] != 5) && (spriteTiles[cy*10+cx] != 6))
				{
					if (currentTile == 6)
					{
						groundTiles[cy*10+cx]=5;
					}
					else if (currentTile == 9)
					{
						groundTiles[cy*10+cx]=8;
					}
					else
					{
						groundTiles[cy*10+cx]=currentTile;
					}
				}
			}

			if ((osl_keys->pressed.triangle) && ((cx > 0) && (cx < 9)) && ((cy > 0) && (cy < 9)))
			{
				spriteTiles[cy*10+cx]=currentSprite;

				if ((currentSprite==5) || (currentSprite==6))
				{
					groundTiles[cy*10+cx]=1;
				}
			}

			if (osl_keys->pressed.circle)
			{
				if (((cy*10+cx)==4) || ((cy*10+cx)==5))
				{
					if (nesw[0]==0)
					{
						nesw[0]=1;
					}
					else
					{
						nesw[0]=0;
					}
				}

				if (((cy*10+cx)==49) || ((cy*10+cx)==59))
				{
					if (nesw[1]==0)
					{
						nesw[1]=1;
					}
					else
					{
						nesw[1]=0;
					}
				}

				if (((cy*10+cx)==94) || ((cy*10+cx)==95))
				{
					if (nesw[2]==0)
					{
						nesw[2]=1;
					}
					else
					{
						nesw[2]=0;
					}
				}

				if (((cy*10+cx)==40) || ((cy*10+cx)==50))
				{
					if (nesw[3]==0)
					{
						nesw[3]=1;
					}
					else
					{
						nesw[3]=0;
					}
				}
			}
		}
		else
		{
			if ((osl_keys->pressed.up) && (ch > 0))
			{
				ch--;
			}

			if ((osl_keys->pressed.down) && (ch < 6))
			{
				ch++;
			}

			if (((osl_keys->pressed.cross) || (osl_keys->pressed.circle)))
			{
				if ((ch==4) || (ch==5))
				{
					while (!chosen)
					{
						oslStartDrawing();
						oslDrawFillRect(0, 0, 480, 272, RGB(32, 32, 32));

						center(64, "Which room number?", 0, RGB(255,255,0));

						sprintf(roomno, "%d", levelno);
						center(116, roomno, 1, RGB(255,255,255));

						oslEndDrawing();
						oslSwapBuffers();
						oslWaitVSync();

						oslReadKeys();

						if (osl_keys->pressed.up)
						{
							levelno-=ROOM_WIDTH;
							if (levelno < 0) levelno=0;
						}

						if (osl_keys->pressed.down)
						{
							levelno+=ROOM_WIDTH;
							if (levelno > ROOM_WIDTH*ROOM_WIDTH) levelno=(ROOM_WIDTH*ROOM_WIDTH)-1;
						}

						if ((osl_keys->pressed.left) && (levelno > 0))
						{
							levelno--;
						}

						if ((osl_keys->pressed.right) && (levelno < maxcases))
						{
							levelno++;
						}

						if ((osl_keys->pressed.cross))
						{
							chosen=1;
						}
					}
				}

			    switch(ch)
			    {
			    	case 0:
			    	case 1:
			    	case 2:
			    		colors[ch]=pickColor(colors[ch]);
			    		break;

			    	case 3:
			    		groundTiles=initTiles();
			    		spriteTiles=initTilesSprite();
			    		for (i=0; i<4; i++) nesw[i]=0;
			    		ch=0;
			    		dispMenu=0;
			    		break;

			    	case 4:
			    		saveRoom(groundTiles, spriteTiles, nesw, levelno);
			    		break;

			    	case 5:
			    		groundTiles=loadRoom(levelno);
			    		spriteTiles=loadSprites(levelno);
			    		nesw=loadDoors(levelno);
			    		loadColors(levelno);
			    		dispMenu=0;
			    		break;

			    	case 6:
			    		quit=1;
			    		break;
			    }

			    chosen=0;
			}
		}

		if (osl_keys->pressed.start)
		{
			if (!dispMenu)
			{
				dispMenu=1;
				ch=0;
			}
			else
			{
				dispMenu=0;
			}
		}
	}

	  for (i=0; i<maxtiles; i++)
	  {
			oslDeleteImage(tiles[i]);
	  }

	  for (i=0; i<maxsprites; i++)
	  {
			oslDeleteImage(sprites[i]);
	  }
}
