/*
 * main.c
 *
 * Copyright  2018 by 240-185
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */

#define _16bit OSL_PF_8888
#define spr_draw oslDrawImageXY
#define ROOM_WIDTH 5
#define MAXSPRITES 4
#define NBTILES 12
#define MAXACTORS 37
#define DEBUG 0
#define MAXSOUNDS 10

#include <oslib/oslib.h>
#include <oslib/libpspmath/pspmath.h>
#include <psputils.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspmpeg.h>
#include <pspdisplay.h>
#include <psprtc.h>
#include <psppower.h>
#include <pspfpu.h>
#include <pspgu.h>
#include <psptypes.h>
#include "common.h"
#include "room.h"
#include "quicksort.h"
#include "define_room.h"
#include "sprite_struct.h"
#include "menu.h"
#include "lveditor.h"
#include <stdio.h>
#include <stdlib.h>

PSP_MODULE_INFO("Odyssey for Panic!", 0, 1, 1);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);
PSP_HEAP_SIZE_KB(-1024);

OSL_IMAGE *wallws[64];
OSL_IMAGE *walles[64];
OSL_IMAGE *grounds[NBTILES];
OSL_IMAGE *doors[16];
OSL_IMAGE *actors[256];
OSL_IMAGE *enemyAsset[3];
OSL_IMAGE *ui[4];

OSL_COLOR colors[3];

OSL_SOUND *sfxs[MAXSOUNDS];

SceKernelUtilsMt19937Context ctx;

float enemyData[3][9];

int *initNESW()
{
	int j;
	static int a[4];

	for (j=0; j<4; j++)
	{
		a[j]=0;
	}

	return a;
}

int *initTables()
{
	int j;
	static int a[100];

	for (j=0; j<100; j++)
	{
		a[j]=0;
	}

	return a;
}

int *initPassThrough()
{
	int j;
	static int a[256];

	for (j=0; j<256; j++)
	{
		a[j]=0;
	}

	return a;
}

OSL_COLOR setAssetColor(int i)
{
	OSL_COLOR c;

	switch(i)
	{
		case 1: c=RGB(255, 208, 132); break;
		case 2:
		case 4: c=RGB(160, 160, 160); break;
		case 3: c=RGB(128, 128, 144); break;
		case 15:
		case 16:
		case 17:
		case 18: c=RGB(192, 192, 192); break;
		case 19:
		case 20:
		case 21:
		case 22:
		case 23:
		case 24:
		case 25:
		case 26:
		case 27:
		case 28:
		case 29:
		case 30:
				c=RGB(255, 183, 112); break;
		case 35:
				c=RGB(255, 0, 0); break;
		default: c=RGB(255, 255, 255); break;
	}

	return c;
}

void init_actor(Sprite *sprs, int i, int actorId, int startingRoom, float x, int y, int z, float speed, int health, OSL_COLOR color, short facing, short grav, short lethal, short pushable)
{
	sprs[i].x=x;
	sprs[i].y=y;
	sprs[i].z=z;
	sprs[i].speed=speed;
	sprs[i].assetId=i;
	sprs[i].room=startingRoom;
	sprs[i].height=actors[actorId]->sizeY;
	sprs[i].color=color;
	sprs[i].health=health;
	sprs[i].isMoving=facing;
	sprs[i].affByGravity=grav;
	sprs[i].actorId=actorId;
	sprs[i].isLethal=lethal;
	sprs[i].isPushable=pushable;
	sprs[i].actualX=2*x+1;
	sprs[i].actualY=y+1;
}

void init_enemies()
{
	int i;

	for (i=0; i<3; i++)
	{
		  enemyData[i][0] = 1.75; //x
		  enemyData[i][1] = 0.5; //y
		  enemyData[i][2] = 0; //z
		  enemyData[i][3] = random(24)+1; //room
		  enemyData[i][4] = 0.0625; //speed
		  enemyData[i][5] = 2*enemyData[0][0]+1; //actualX
		  enemyData[i][6] = enemyData[0][1]+1; //actualY
		  enemyData[i][7] = 0; //hasReachedCenter
		  enemyData[i][8] = 5; //hasChosenDir
	}
}

void load_actor_image(int i)
{
	char bouffeur[30];

	if (i > 0)
	{
		if (i<10) sprintf(bouffeur, "./gfx/actor0%d.png", i); else sprintf(bouffeur, "./gfx/actor%d.png", i);
		actors[i] = oslLoadImageFilePNG(bouffeur, OSL_IN_RAM, OSL_PF_8888);
		actors[i]->centerX = actors[i]->sizeX/2;
		actors[i]->centerY = actors[i]->sizeY-8;
	}
	else
	{
		actors[i] = oslLoadImageFilePNG("./gfx/roburi.png", OSL_IN_RAM, OSL_PF_8888);
	}

}

void debug(const char *text)
{
	  oslStartDrawing();
	  oslDrawFillRect(0, 0, 480, 272, RGB(0, 0, 0));
	  center(136, text, 2, RGB(255, 255, 0));
	  oslEndDrawing();
	  oslSwapBuffers();
	  oslWaitKey();
}

void drawUI(int lives, int health, int noise, int fear)
{
	char bouffeur[3];

	oslSetAlpha(OSL_FX_TINT, RGB(255, 0, 0));
	spr_draw(ui[1], 8, 200);
	oslDrawRect(24, 200, 128, 208, RGB(255, 0, 0));
	oslDrawFillRect(26, 202, 26+health, 207, RGB(255, 0, 0));

	oslSetAlpha(OSL_FX_TINT, RGB(0, 255, 255));
	spr_draw(ui[2], 352, 200);
	oslSetAlpha(OSL_FX_DEFAULT, 0);
	oslDrawRect(368, 200, 472, 208, RGB(0, 255, 255));
	oslDrawFillRect(368, 202, 368+fear, 207, RGB(0, 255, 255));

	oslSetAlpha(OSL_FX_TINT, RGB(255, 128, 0));
	spr_draw(ui[3], 352, 224);

	oslSetAlpha(OSL_FX_DEFAULT, 0);
	oslDrawRect(368, 224, 472, 232, RGB(255, 128, 0));
	oslDrawFillRect(370, 226, 370+noise, 231, RGB(255, 128, 0));

	spr_draw(ui[0], 8, 224);

	sprintf(bouffeur, "%d", lives);
	setFont(1);
	oslDrawString(32, 232, bouffeur);
}

void game()
{
	  int skipframe=0, quit=1, i=0, val=0, currentRoom=12, jumpframe=-1, predict_val=0, nbactors=MAXACTORS, nbsounds=MAXSOUNDS, nbtiles=NBTILES;
	  int lives=9, health=100, noise=0, fear=0, deathcounter=0;

	  unsigned long int counter=0;

	  float robx=0.25, roby=1, predict_robx=0, predict_roby=0, coeff=0.25, robz3d=0, robxentry=robx, robyentry=roby;
	  float robx3d, roby3d, actx3d, acty3d;

	  int nme_goes[3]={0, 0, 0};

	  short nesw_overview[ROOM_WIDTH*ROOM_WIDTH];

	  short walking=1, facing=1, upanddown=0, j, u, v, w, as, drawn=0, rebond=1, nbAnimAsset=0, lights=1;
	  short isBossDead=0;

	  int *room, *decorum, *nesw, *passThrough;

	  char bouffeur[20];
	  short treasures[2]={0,0};
	  short deposited[2]={0,0};

	  OSL_SOUND *inmaze = oslLoadSoundFileMOD("./sfx/creepy.xm", OSL_FMT_NONE);
	  OSL_IMAGE *torch  = oslLoadImageFilePNG("./gfx/torch.png", OSL_IN_RAM, OSL_PF_8888);
	  OSL_IMAGE *reaper_eyes = oslLoadImageFilePNG("./gfx/reaper_eyes.png", OSL_IN_RAM, OSL_PF_8888);
	  OSL_IMAGE *dead = oslLoadImageFilePNG("./gfx/dead.png", OSL_IN_RAM, OSL_PF_8888);
	  reaper_eyes->centerX = 16;
	  reaper_eyes->centerY = 24;

	  oslImageSetRotCenter(torch);

	  loadColors(currentRoom);
	  Sprite sprs[MAXSPRITES];

	  for (i=0; i<4; i++)
	  {
		  sprintf(bouffeur, "./gfx/ui%d.png", i);
		  ui[i]=oslLoadImageFilePNG(bouffeur, OSL_IN_RAM, OSL_PF_8888);
	  }

	  for (i=0; i<nbsounds; i++)
	  {
		  sprintf(bouffeur, "./sfx/sfx%d.wav", i);
		  sfxs[i]=oslLoadSoundFileWAV(bouffeur, OSL_FMT_NONE);
	  }

	  for (i=0; i<nbactors; i++)
	  {
		  load_actor_image(i);
	  }

	  for (u=0; u<3; u++)
	  {
		  for (v=0; v<5; v++)
		  {
			  enemyData[u][v]=0;
		  }
	  }

	  passThrough=initPassThrough();
	  *(passThrough+5)=1;
	  *(passThrough+9)=1;
	  *(passThrough+10)=1;
	  *(passThrough+32)=1;
	  *(passThrough+34)=1;
	  *(passThrough+35)=1;

	  enemyAsset[0] = oslLoadImageFilePNG("./gfx/reaper.png", OSL_IN_RAM, OSL_PF_8888);
	  enemyAsset[1] = oslLoadImageFilePNG("./gfx/ghoul.png", OSL_IN_RAM, OSL_PF_8888);
	  enemyAsset[2] = oslLoadImageFilePNG("./gfx/banshee.png", OSL_IN_RAM, OSL_PF_8888);

	  init_enemies();

	  for (i=0; i<3; i++)
	  {
		  enemyAsset[i]->centerX = 16;
		  enemyAsset[i]->centerY = 24;
	  }

	  setFont(1);

		wallws[0] = oslLoadImageFilePNG("./gfx/wallw01.png", OSL_IN_RAM, OSL_PF_8888);
		walles[0] = oslLoadImageFilePNG("./gfx/walle01.png", OSL_IN_RAM, OSL_PF_8888);

		wallws[0]->centerX=0;
		wallws[0]->centerY=wallws[0]->sizeY-56;
		walles[0]->centerX=0;
		walles[0]->centerY=walles[0]->sizeY-56;

		for (i=0; i<nbtiles; i++)
		{
			if (i<10) sprintf(bouffeur, "./gfx/grd0%d.png", i); else sprintf(bouffeur, "./gfx/grd%d.png", i);
			grounds[i] = oslLoadImageFilePNG(bouffeur, OSL_IN_RAM, OSL_PF_8888);

			if ((i == 8) || (i == 9))
			{
				grounds[i]->centerX=0;
				grounds[i]->centerY=64;
			}
		}

		doors[0] = oslLoadImageFilePNG("./gfx/door01.png", OSL_IN_RAM, OSL_PF_8888);
		doors[1] = oslLoadImageFilePNG("./gfx/door02.png", OSL_IN_RAM, OSL_PF_8888);

		for (i=0; i<(ROOM_WIDTH*ROOM_WIDTH); i++)
		{
			nesw_overview[i]=0;
			nesw=loadDoors(i);

			if ((nesw[0]) && (i != 5))  nesw_overview[i]+=1;
			if ((nesw[1]) && (i != 24)) nesw_overview[i]+=2;
			if (nesw[2]) nesw_overview[i]+=4;
			if ((nesw[3]) && (i != 1))  nesw_overview[i]+=8;

			nesw_overview[0]=6;
		}

		room=loadRoom(currentRoom);
		decorum=loadSprites(currentRoom);
		nesw=loadDoors(currentRoom);

		oslPlaySound(inmaze, 7);

	  while (quit < 2)
	  {

		if ((currentRoom==0) && (isBossDead))
		{
			*(room+12)=9;
		}

		if (!skipframe)
		{
			if (quit==1)
			{
				if (upanddown > 0)
				{
					if (upanddown == 1)
					{
						walking=1;
						jumpframe+=2;
						robz3d=(jumpframe<16?jumpframe:16);
						if (jumpframe==17)
						{
							upanddown=2;
						}
					}
					else
					{
						walking=0;
						jumpframe-=2;
						robz3d=(jumpframe<0?0:jumpframe);
						if (jumpframe==-1)
						{
							upanddown=0;
							rebond=0;
						}
					}
				}
				else
				{
					robz3d=0;
					walking=1;
				}

				init_actor(sprs, 0, 0, currentRoom, robx, roby, robz3d, 0.25, health, RGB(255, 255, 255), facing, 1, 0, 0);

				sprs[0].x=robx;
				sprs[0].y=roby;
				sprs[0].z=robz3d;
				sprs[0].actualX = 2*robx+1;
				sprs[0].actualY = roby+1;

				oslStartDrawing();
				oslDrawFillRect(0, 0, 480, 272, RGB(0,0,0));

				draw_room_door_bg(doors[0], doors[1], colors[2], nesw);
				draw_room_wall(wallws[0], walles[0], colors[0], nesw);
				draw_room_ground(room, colors[1]);

				drawn=0;

				for (u=0; u<10; u++)
				{
					for (v=0; v<10; v++)
					{
						w=v*10+u;
						val = (int)sprs[0].actualY*10+(int)sprs[0].actualX;

						robx3d = 32*(robx)-16*(roby)+241;
						roby3d = 16*(robx)+8*(roby)+82-((int)robz3d);

						if (val == w)
						{
							if (!deathcounter)
							{
								draw_roburi(actors[0], sprs[0].color, robx3d, roby3d, walking, facing, counter);
							}
							else
							{
								nbAnimAsset=((dead->sizeX)/24)-1;
								as=(deathcounter%30);
								oslSetImageTileSize(dead, 24*(as/(30/nbAnimAsset)), 0, 24, dead->sizeY);
								dead->centerX = 12;
								dead->centerY = 24;
								draw_actor(dead, sprs[0].color, robx3d, roby3d, walking, facing);
							}

							drawn=1;
						}

						for (j=0; j<3; j++)
						{
							if (enemyData[j][3] == currentRoom)
							{
								if (((int)enemyData[j][6]*10+(int)enemyData[j][5]) == w)
								{
									actx3d = 32*(enemyData[j][0])-16*(enemyData[j][1])+241;
									acty3d = 16*(enemyData[j][0])+8*(enemyData[j][1])+82;
									draw_actor(enemyAsset[j], RGB(128, 128, 128), actx3d, acty3d, 0, 0);

									if (j==0)
									{
										spr_draw(reaper_eyes, actx3d, acty3d);
									}
								}
							}
						}

						if ((currentRoom==24) && (!isBossDead))
						{
							*(decorum+46) = 8;
							*(decorum+56) = 8;
						}

						if (((*(room+w) == 10) && (deposited[0])))
						{
							*(decorum+w)=34;
						}

						if (((*(room+w) == 11) && (deposited[1])))
						{
							*(decorum+w)=35;
						}

						if ((*(decorum+w) == 34) && (deposited[0] || treasures[0] || isBossDead) && (currentRoom != 0))
						{
							*(decorum+w)=0;
						}

						if ((*(decorum+w) == 35) && (deposited[1] || treasures[1] || isBossDead) && (currentRoom != 0))
						{
							*(decorum+w)=0;
						}

						if (*(decorum+w) > 0)
						{
							actx3d = 32*(((u-0.5)/2)+0.25)-16*(v-0.5)+232;
							acty3d = 16*(((u-0.5)/2)+0.25)+8*(v-0.5)+84; //-((int)sprs[1].z)

							  if (actors[*(decorum+w)]->sizeX > 32)
						  	  {
									nbAnimAsset=(actors[*(decorum+w)]->sizeX)/32;
									as=(counter%60);
									oslSetImageTileSize(actors[*(decorum+w)], 32*(as/(60/nbAnimAsset)), 0, 32, actors[*(decorum+w)]->sizeY);
									actors[*(decorum+w)]->centerX=16;
									actors[*(decorum+w)]->centerY=actors[*(decorum+w)]->sizeY-8;
						  	  }

 							draw_actor(actors[*(decorum+w)], setAssetColor(*(decorum+w)), actx3d, acty3d, 0, 0);
						}

						if (passThrough[*(decorum+w)] > 0)
						{
							if (val == w)
							{
								draw_roburi(actors[0], sprs[0].color, robx3d, roby3d, walking, facing, counter);
							}
						}
					}
				}

				if ((!drawn) && (!deathcounter))
				{
					robx3d = 32*(robx)-16*(roby)+240;
					roby3d = 16*(robx)+8*(roby)+80-((int)robz3d);
					draw_roburi(actors[0], sprs[0].color, robx3d, roby3d, walking, facing, counter);
				}

				if ((*(decorum+val) == 34) && (!(deposited[0])))
				{
					treasures[0]=1;
					*(decorum+val) = 0;
				}

				if ((*(decorum+val) == 35) && (!(deposited[1])))
				{
					treasures[1]=1;
					*(decorum+val) = 0;
				}

				if ((*(room+val) == 10) && (!deposited[0]) && (treasures[0]==1))
				{
					deposited[0]=1;
					*(decorum+val) = 34;
					treasures[0]=0;
				}

				if ((*(room+val) == 11) && (!deposited[1]) && (treasures[1]==1))
				{
					deposited[1]=1;
					*(decorum+val) = 35;
					treasures[1]=0;
				}

				oslSetAlpha(OSL_FX_DEFAULT, 0);
				draw_room_door_fg(doors[0], doors[1], colors[2], nesw);

				if ((nesw[3]) && (sprs[0].actualX < 1.13))
				{
					if ((sprs[0].actualY >= 4.75) && (sprs[0].actualY <= 5.25))
					{
						if (facing==3)
						{
							currentRoom--;

							room=loadRoom(currentRoom);
							decorum=loadSprites(currentRoom);
							nesw=loadDoors(currentRoom);
							loadColors(currentRoom);

							robx=3.75;
							robxentry=robx;
							robyentry=roby;

							for (w=0; w<3; w++)
							{
								if ((enemyData[w][3]-currentRoom)*(enemyData[w][3]-currentRoom) < 2)
								{
									oslPlaySound(sfxs[w], w+1);
								}

								enemyData[w][7] = 0; //hasReachedCenter
								enemyData[w][8] = 5; //hasChosenDir
							}
						}
					}
					else
					{
						if ((sprs[0].actualY > 4) && (sprs[0].actualY < 4.75) && (facing==3))
						{
							roby+=0.25;
						}
						if ((sprs[0].actualY > 5.25) && (sprs[0].actualY < 6) && (facing==3))
						{
							roby-=0.25;
						}
					}
				}

				if ((nesw[1]) && (sprs[0].actualX > 8.87))
				{
					if ((sprs[0].actualY >= 4.75) && (sprs[0].actualY <= 5.25))
					{
						if (facing==1)
						{
							currentRoom++;

							if (currentRoom < 25)
							{
								room=loadRoom(currentRoom);
								decorum=loadSprites(currentRoom);
								nesw=loadDoors(currentRoom);
								loadColors(currentRoom);
							}
							else
							{
								if (currentRoom==25)
								{
									debug("Yay, you won.");
									debug("What did you expect?");
									debug("That's enough for today.");
									quit=2;
								}
							}

							robx=0.25;
							robxentry=robx;
							robyentry=roby;

							for (w=0; w<3; w++)
							{
								if ((enemyData[w][3]-currentRoom)*(enemyData[w][3]-currentRoom) < 2)
								{
									oslPlaySound(sfxs[w], w+1);
								}

								enemyData[w][7] = 0; //hasReachedCenter
								enemyData[w][8] = 5; //hasChosenDir
							}
						}
					}
					else
					{
						if ((sprs[0].actualY > 4) && (sprs[0].actualY < 4.75) && (facing==1))
						{
							roby+=0.25;
						}
						if ((sprs[0].actualY > 5.25) && (sprs[0].actualY < 6) && (facing==1))
						{
							roby-=0.25;
						}
					}

				}

				if ((nesw[2]) && (sprs[0].actualY > 8.87))
				{
					if ((sprs[0].actualX >= 4.75) && (sprs[0].actualX <= 5.25))
					{
						if (facing==2)
						{
							currentRoom+=ROOM_WIDTH;

								room=loadRoom(currentRoom);
								decorum=loadSprites(currentRoom);
								nesw=loadDoors(currentRoom);
								loadColors(currentRoom);

							roby=0.50;
							robxentry=robx;
							robyentry=roby;

							for (w=0; w<3; w++)
							{
								if ((enemyData[w][3]-currentRoom)*(enemyData[w][3]-currentRoom) < 2)
								{
									oslPlaySound(sfxs[w], w+1);
								}

								enemyData[w][7] = 0; //hasReachedCenter
								enemyData[w][8] = 5; //hasChosenDir
							}
						}
					}
					else
					{
						if ((sprs[0].actualX > 4) && (sprs[0].actualX < 4.75) && (facing==2))
						{
							robx+=0.125;
						}
						if ((sprs[0].actualX > 5.25) && (sprs[0].actualX < 6) && (facing==2))
						{
							robx-=0.125;
						}
					}

				}

				if ((nesw[0]) && (sprs[0].actualY < 1.13))
				{
					if ((sprs[0].actualX >= 4.75) && (sprs[0].actualX <= 5.25))
					{
						if (facing==0)
						{
							currentRoom-=ROOM_WIDTH;

								room=loadRoom(currentRoom);
								decorum=loadSprites(currentRoom);
								nesw=loadDoors(currentRoom);
								loadColors(currentRoom);

							roby=7.50;
							robxentry=robx;
							robyentry=roby;

							for (w=0; w<3; w++)
							{
								if ((enemyData[w][3]-currentRoom)*(enemyData[w][3]-currentRoom) < 2)
								{
									oslPlaySound(sfxs[w], w+1);
								}

								enemyData[w][7] = 0; //hasReachedCenter
								enemyData[w][8] = 5; //hasChosenDir
							}
						}
					}
					else
					{
						if ((sprs[0].actualX > 4) && (sprs[0].actualX < 4.75) && (facing==0))
						{
							robx+=0.125;
						}
						if ((sprs[0].actualX > 5.25) && (sprs[0].actualX < 6) && (facing==0))
						{
							robx-=0.125;
						}
					}

				}

		        if ((2*robx <= -1) || (roby <= -1))
		        {
					draw_room_door_bg(doors[0], doors[1], colors[2], nesw);
					draw_room_wall(wallws[0], walles[0], colors[0], nesw);
		        }
		        if ((2*robx > 8) || (roby > 8))
		        {
					draw_room_door_fg(doors[0], doors[1], colors[2], nesw);
		        }


		        if (!lights)
		        {
		        	oslDrawFillRect(0, 0, 480, 272, RGBA(0, 0, 0, 192));
		        	if ((!(counter%7)) && (fear < 100)) fear++;
		        }
		        else
		        {
		        	oslDrawFillRect(0, 0, robx3d-64, 272, RGBA(0, 0, 0, 208));
		        	oslDrawFillRect(robx3d-64, 0, robx3d+64, roby3d-64, RGBA(0, 0, 0, 208));
		        	oslDrawFillRect(robx3d+64, 0, 480, 272, RGBA(0, 0, 0, 208));
		        	oslDrawFillRect(robx3d-64, roby3d+64, robx3d+64, 272, RGBA(0, 0, 0, 208));
		        	oslSetAlpha(OSL_FX_ALPHA, 208);
		        	spr_draw(torch, robx3d, roby3d);
		        	oslSetAlpha(OSL_FX_DEFAULT, 0);
		        }

				for (j=0; j<3; j++)
				{
					oslSetAlpha(OSL_FX_DEFAULT, 0);

					if ((enemyData[j][3] == currentRoom) && (!j))
					{
						if (((int)enemyData[j][6]*10+(int)enemyData[j][5]) == w)
						{
							actx3d = 32*(enemyData[j][0])-16*(enemyData[j][1])+241;
							acty3d = 16*(enemyData[j][0])+8*(enemyData[j][1])+82;

							spr_draw(reaper_eyes, actx3d, acty3d);
						}
					}
				}


		        if ((deposited[0]) && (deposited[1]) && (!isBossDead))
		        {
		        	oslPlaySound(sfxs[8], 6);
		        	isBossDead=1;
		        }

		        if ((currentRoom==0) && (!isBossDead) && (lights) && (health > 0))
		        {
						oslDrawLine(286, 57, robx3d, roby3d-12, RGBA(255,255,0,random(255)));
						oslDrawLine(302, 58, robx3d, roby3d-12, RGBA(255,255,0,random(255)));

		        	health--;
		        }

		        if ((treasures[0]) && (!(deposited[0]))) spr_draw(actors[34], 90, 32);
		        if ((treasures[1]) && (!(deposited[1]))) spr_draw(actors[35], 358, 32);

				if (DEBUG)
				{
					sprintf(bouffeur, "salle no. : %d", currentRoom);
					oslDrawString(0, 0, bouffeur);

					/*sprintf(bouffeur, "%d", deathcounter);
					oslDrawString(0, 40, bouffeur);

					sprintf(bouffeur, "free RAM: %d", (int)oslGetRamStatus().maxAvailable);
					oslDrawString(0, 264, bouffeur);

					sprintf(bouffeur, "no dalle : %d", val);
					oslDrawString(0, 24, bouffeur);

					sprintf(bouffeur, "val dalle : %d", *(room+val));
					oslDrawString(0, 32, bouffeur);*/

					sprintf(bouffeur, "dalles NME : %d %d %d", ((int)enemyData[0][6]*10+(int)enemyData[0][5]), ((int)enemyData[1][6]*10+(int)enemyData[1][5]), ((int)enemyData[2][6]*10+(int)enemyData[2][5]));
					oslDrawString(0, 8, bouffeur);

					sprintf(bouffeur, "salles NME : %d %d %d", ((int)enemyData[0][3]), ((int)enemyData[1][3]), ((int)enemyData[2][3]));
					oslDrawString(0, 16, bouffeur);
				}
				drawUI(lives, health, noise, fear);

				oslEndDrawing();
			}
		}

        oslEndFrame();
        counter++;
        skipframe = oslSyncFrame();

        for(u=0; u<3; u++)
        {
        //beginFor
        if (deathcounter == 0)
        {
			if (((u==0) && ((!lights) || (enemyData[0][3] != currentRoom)))
			||  ((u==1) && ((noise >= 0) || (enemyData[1][3] != currentRoom)))
			||  ((u==2) && ((fear  >= 0) || (enemyData[2][3] != currentRoom))))
			{
				if (enemyData[u][7]==0)
				{
					if ((2) <= enemyData[u][0])
					{
						enemyData[u][0]=enemyData[u][0]-((enemyData[u][4])/2);
					}

					if ((2) >= enemyData[u][0])
					{
						enemyData[u][0]=enemyData[u][0]+((enemyData[u][4])/2);
					}

					if ((3.8) >= enemyData[u][1])
					{
						enemyData[u][1]=enemyData[u][1]+((enemyData[u][4]));
					}

					if ((3.8) <= enemyData[u][1])
					{
						enemyData[u][1]=enemyData[u][1]-((enemyData[u][4]));
					}

					enemyData[u][5] = 2*enemyData[u][0]+1;
					enemyData[u][6] = enemyData[u][1]+1;

					if ((enemyData[u][0] >= 1.9) && (enemyData[u][0] <= 2.1)
					 && (enemyData[u][1] >= 3.7) && (enemyData[u][1] <= 3.9))
					{
						enemyData[u][0]=2;
						enemyData[u][1]=3.8;
						enemyData[u][7]=1;
					}
				}
				else
				{
					if (enemyData[u][8]==5)
					{
						if (((u==0) && (!lights)) || ((u==1) && (noise < 45)) || ((u==2) && (fear < 45)))
						{
							nme_goes[u]=random(4);
						}
						else
						{
							if (enemyData[u][3]-currentRoom >= 1) nme_goes[u]=1;
							if (enemyData[u][3]-currentRoom <= -1) nme_goes[u]=3;
							if (enemyData[u][3]-currentRoom >= 5) nme_goes[u]=0;
							if (enemyData[u][3]-currentRoom <= -5) nme_goes[u]=2;
						}

						while ((((nesw_overview[((int)enemyData[u][3])] & 1)==0) && (nme_goes[u]==0)) ||
								(((nesw_overview[((int)enemyData[u][3])] & 2)==0) && (nme_goes[u]==1)) ||
								(((nesw_overview[((int)enemyData[u][3])] & 4)==0) && (nme_goes[u]==2)) ||
								(((nesw_overview[((int)enemyData[u][3])] & 8)==0) && (nme_goes[u]==3)))
						{
							nme_goes[u]=random(4);
						}

						enemyData[u][8]=(float)(nme_goes[u]);
					}


					switch(nme_goes[u])
					{
						case 3:
							enemyData[u][0]=enemyData[u][0]-((enemyData[u][4])/2);
							break;

						case 1:
							enemyData[u][0]=enemyData[u][0]+((enemyData[u][4])/2);
							break;

						case 0:
							enemyData[u][1]=enemyData[u][1]-((enemyData[u][4]));
							break;

						case 2:
							enemyData[u][1]=enemyData[u][1]+((enemyData[u][4]));
							break;
					}

					if (enemyData[u][5] < 1.13)
					{
						if ((enemyData[u][6] >= 4) && (enemyData[u][6] <= 5))
						{
								enemyData[u][3]--;
								enemyData[u][0]=3.75;
								enemyData[u][7]=0;
								enemyData[u][8]=5;
						}
					}

					if (enemyData[u][5] > 8.87)
					{
						if ((enemyData[u][6] >= 4) && (enemyData[u][6] <= 5))
						{
								enemyData[u][3]++;
								enemyData[u][0]=0.25;
								enemyData[u][7]=0;
								enemyData[u][8]=5;
						}
					}

					if (((int)enemyData[u][6]*10+(int)enemyData[u][5]) < 10)
					{
						if ((enemyData[u][5] >= 4) && (enemyData[u][5] <= 5))
						{
								enemyData[u][3]-=ROOM_WIDTH;
								enemyData[u][1]=7.5;
								enemyData[u][7]=0;
								enemyData[u][8]=5;
						}
					}

					if (((int)enemyData[u][6]*10+(int)enemyData[u][5]) > 89)
					{
						if ((enemyData[u][5] >= 4) && (enemyData[u][5] <= 5))
						{
								enemyData[u][3]+=ROOM_WIDTH;
								enemyData[u][1]=0.5;
								enemyData[u][7]=0;
								enemyData[u][8]=5;
						}
					}

					if ((enemyData[u][3]-currentRoom)*(enemyData[u][3]-currentRoom) < 2)
					{
						oslPlaySound(sfxs[u], u+1);
					}

					enemyData[u][5] = 2*enemyData[u][0]+1;
					enemyData[u][6] = enemyData[u][1]+1;
				}
			}

			if ((enemyData[u][3] == currentRoom))
			{
				if (u==2)
				{
					if ((!(counter%4)) && (fear < 100)) fear++;
				}

				if (((lights) && (u==0)) || (u > 0))
					{
					if ((robx) < enemyData[u][0])
					{
						enemyData[u][0]=enemyData[u][0]-((enemyData[u][4])/2);
					}

					if ((robx) > enemyData[u][0])
					{
						enemyData[u][0]=enemyData[u][0]+((enemyData[u][4])/2);
					}

					if ((roby) > enemyData[u][1])
					{
						enemyData[u][1]=enemyData[u][1]+((enemyData[u][4]));
					}

					if ((roby) < enemyData[u][1])
					{
						enemyData[u][1]=enemyData[u][1]-((enemyData[u][4]));
					}

					enemyData[u][5] = 2*enemyData[u][0]+1;
					enemyData[u][6] = enemyData[u][1]+1;
				}
			}
         } // endFor
       }
        switch (*(room+val))
        {
        case 4:
        	health=0;
        break;

        case 3:
    		if (oslGetSoundChannel(sfxs[6]) != 6)
    		{
    			oslPlaySound(sfxs[6], 6);
    		}

        	if (!(counter%4))
        	{
        		health-=2;
        		fear++;
        		noise++;
        	}
        	break;

        	case 5:
        		if (!rebond)
        		{
        			*(room+val)=6;
        			rebond=1;

        			for(i=0; i<100; i++)
        			{
        				switch (*(decorum+i))
        				{
        					case 5:
        					*(decorum+i)=6;
        					break;

        					case 6:
        					*(decorum+i)=5;
        					break;
        				}
        			}
        		}
        		break;

        	case 6:
        		if (!rebond)
        		{
        			*(room+val)=5;
        			rebond=1;

        			for(i=0; i<100; i++)
        			{
        				switch (*(decorum+i))
        				{
        					case 5:
        					*(decorum+i)=6;
        					break;

        					case 6:
        					*(decorum+i)=5;
        					break;
        				}
        			}
        		}
        		break;

        }

        oslReadKeys();

        if (*(room+val) == 2)
        {
        	if (!(counter%3))
        	{
        		if (fear < 100) fear++;
        	}

        	switch(facing)
        	{
        		case 0:
        			if (sprs[0].actualY > 1.13)
					{
						if ((osl_pad.analogY < -64) && (osl_pad.analogX > 64)) coeff=0.5; else coeff=0.25;
						predict_robx=robx;
						predict_roby=roby-(sprs[0].speed)*coeff;
						predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

						if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
						{
							if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
							roby-=(sprs[0].speed)*coeff;

							if (coeff == 0.5)
							{
								if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
								if ((!(counter%5)) && (noise < 100)) noise+=2;
							}
						}
						else
						{
							if (oslGetSoundChannel(sfxs[3]) != 0)
							{
								oslPlaySound(sfxs[3], 0);
								noise += 10;
								if (noise > 100) noise=100;
							}
						}
					}
        			break;

        		case 2:
    				if (sprs[0].actualY < 8.87)
    				{
    					if ((osl_pad.analogY > 64) && (osl_pad.analogX < -64)) coeff=0.5; else coeff=0.25;
    					predict_robx=robx;
    					predict_roby=roby+(sprs[0].speed)*coeff;
    					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

    					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
    					{
    						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
    						roby+=(sprs[0].speed)*coeff;

							if (coeff == 0.5)
							{
								if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
								if ((!(counter%5)) && (noise < 100)) noise+=2;
							}
    					}
						else
						{
							if (oslGetSoundChannel(sfxs[3]) != 0)
							{
								oslPlaySound(sfxs[3], 0);
								noise += 10;
								if (noise > 100) noise=100;
							}
						}
    				}
    				break;

        		case 3:
    				if (sprs[0].actualX > 1.13)
    				{
    					if ((osl_pad.analogY < -64) && (osl_pad.analogX < -64)) coeff=0.5; else coeff=0.25;
    					predict_roby=roby;
    					predict_robx=robx-((sprs[0].speed)/2)*coeff;
    					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

    					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
    					{
    						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
    						robx-=((sprs[0].speed)/2)*coeff;

							if (coeff == 0.5)
							{
								if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
								if ((!(counter%5)) && (noise < 100)) noise+=2;
							}
    					}
						else
						{
							if (oslGetSoundChannel(sfxs[3]) != 0)
							{
								oslPlaySound(sfxs[3], 0);
								noise += 10;
								if (noise > 100) noise=100;
							}
						}
    				}
    				break;

        		case 1:
    				if (sprs[0].actualX < 8.87)
    				{
    					if ((osl_pad.analogY > 64) && (osl_pad.analogX > 64)) coeff=0.5; else coeff=0.25;
    					predict_roby=roby;
    					predict_robx=robx+((sprs[0].speed)/2)*coeff;
    					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

    					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
    					{
    						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
    						robx+=((sprs[0].speed)/2)*coeff;

							if (coeff == 0.5)
							{
								if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
								if ((!(counter%5)) && (noise < 100)) noise+=2;
							}
    					}
						else
						{
							if (oslGetSoundChannel(sfxs[3]) != 0)
							{
								oslPlaySound(sfxs[3], 0);
								noise += 10;
								if (noise > 100) noise=100;
							}
						}
    				}
    				break;
        	}
        }
        else
        {
			if ((osl_pad.analogY < -16) && (osl_pad.analogX > 16) && (deathcounter == 0)) //ur
			{
				facing=0;

				if (sprs[0].actualY > 1.13)
				{
					if ((osl_pad.analogY < -64) && (osl_pad.analogX > 64)) coeff=0.5; else coeff=0.25;
					predict_robx=robx;
					predict_roby=roby-(sprs[0].speed)*coeff;
					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
					{
						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
						roby-=(sprs[0].speed)*coeff;

						if (coeff == 0.5)
						{
							if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
							if ((!(counter%5)) && (noise < 100)) noise+=2;
						}
					}
					else
					{
						if (oslGetSoundChannel(sfxs[3]) != 0)
						{
							oslPlaySound(sfxs[3], 0);
							noise += 10;
							if (noise > 100) noise=100;
						}
					}
				}
			}

			if ((osl_pad.analogY > 16) && (osl_pad.analogX < -16) && (deathcounter == 0)) //ul
			{
				facing=2;

				if (sprs[0].actualY < 8.87)
				{
					if ((osl_pad.analogY > 64) && (osl_pad.analogX < -64)) coeff=0.5; else coeff=0.25;
					predict_robx=robx;
					predict_roby=roby+(sprs[0].speed)*coeff;
					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
					{
						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
						roby+=(sprs[0].speed)*coeff;

						if (coeff == 0.5)
						{
							if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
							if ((!(counter%5)) && (noise < 100)) noise+=2;
						}
					}
					else
					{
						if (oslGetSoundChannel(sfxs[3]) != 0)
						{
							oslPlaySound(sfxs[3], 0);
							noise += 10;
							if (noise > 100) noise=100;
						}
					}
				}

			}

			if ((osl_pad.analogY < -16) && (osl_pad.analogX < -16) && (deathcounter == 0)) //dl
			{
				facing=3;

				if (sprs[0].actualX > 1.13)
				{
					if ((osl_pad.analogY < -64) && (osl_pad.analogX < -64)) coeff=0.5; else coeff=0.25;
					predict_roby=roby;
					predict_robx=robx-((sprs[0].speed)/2)*coeff;
					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
					{
						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
						robx-=((sprs[0].speed)/2)*coeff;

						if (coeff == 0.5)
						{
							if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
							if ((!(counter%5)) && (noise < 100)) noise+=2;
						}
					}
					else
					{
						if (oslGetSoundChannel(sfxs[3]) != 0)
						{
							oslPlaySound(sfxs[3], 0);
							noise += 10;
							if (noise > 100) noise=100;
						}
					}
				}
			}

			if ((osl_pad.analogY > 16) && (osl_pad.analogX > 16) && (deathcounter == 0)) //dr
			{
				facing=1;

				if (sprs[0].actualX < 8.87)
				{
					if ((osl_pad.analogY > 64) && (osl_pad.analogX > 64)) coeff=0.5; else coeff=0.25;
					predict_roby=roby;
					predict_robx=robx+((sprs[0].speed)/2)*coeff;
					predict_val=((int)(predict_roby+1))*10+((int)(2*predict_robx+1));

					if ((*(decorum+predict_val) == 0) || (passThrough[*(decorum+predict_val)]==1))
					{
						if ((val != predict_val) && (robz3d==0)) rebond=0; else rebond=1;
						robx+=((sprs[0].speed)/2)*coeff;

						if (coeff == 0.5)
						{
							if (oslGetSoundChannel(sfxs[9]) != 5) oslPlaySound(sfxs[9], 5);
							if ((!(counter%5)) && (noise < 100)) noise+=2;
						}
					}
					else
					{
						if (oslGetSoundChannel(sfxs[3]) != 0)
						{
							oslPlaySound(sfxs[3], 0);
							noise += 10;
							if (noise > 100) noise=100;
						}
					}
				}
			}

			if ((osl_keys->pressed.square) && (jumpframe<0) && (deathcounter == 0))
			{
				upanddown=1;
			}
        }

		if ((osl_keys->pressed.triangle)  && (deathcounter == 0))
		{
			if (!lights)
			{
				lights=1;
			}
			else
			{
				lights=0;
				enemyData[0][7]=0;
				enemyData[0][8]=5;
			}
		}

		if (osl_keys->pressed.start)
		{
			quit++;
		}

		for (i=0; i<3; i++)
		{
			if ((((int)enemyData[i][6]*10+(int)enemyData[i][5]) == val) && ((int)enemyData[i][3] == currentRoom))
			{
				if ((!(counter%4)) && (health > 0)) health-=5;

				if ((oslGetSoundChannel(sfxs[5]) != 6) && (deathcounter == 0))
				{
					oslPlaySound(sfxs[5], 6);
				}
			}
		}

		if (fear > 100) fear=100;
		if (noise > 100) noise=100;

		if ((health <= 0) || (deathcounter > 0))
		{
			health=0;
			deathcounter++;

			if (deathcounter == 1) oslPlaySound(sfxs[4], 7);

			if (deathcounter == 90)
			{
				robx=robxentry;
				roby=robyentry;

				deathcounter=0;
				health=100;
				lives--;

				init_enemies();

				for (i=0; i<2; i++)
				{
					while (enemyData[i][3]==currentRoom)
					{
						enemyData[i][3]=random(24)+1;
					}
				}

				if (lives < 0) quit=2; else oslPlaySound(inmaze, 7);
			}
		}

		if (!(counter%30))
		{
			if (health < 100) health++;
		}

		if (!(counter%8))
		{
			if (fear > 0) fear--;
		}

		if (!(counter%8))
		{
			if (noise > 0) noise--;
		}
	  }

	  for (i=0; i<2; i++)
	  {
			oslDeleteImage(doors[i]);
	  }

  for (i=0; i<nbtiles; i++)
	  {
		  	oslDeleteImage(grounds[i]);
	  }

	  oslDeleteImage(walles[0]);
	  oslDeleteImage(wallws[0]);

	  for(i=0; i<3; i++) oslDeleteImage(enemyAsset[i]);

	  oslDeleteImage(torch);
	  oslDeleteImage(reaper_eyes);
	  oslDeleteImage(dead);

	  for (i=0; i<nbactors; i++)
	  {
		  oslDeleteImage(actors[i]);
	  }

	  for (i=0; i<nbsounds; i++)
	  {
		  oslDeleteSound(sfxs[i]);
	  }

	  for (i=0; i<4; i++)
	  {
		  oslDeleteImage(ui[i]);
	  }

	  oslDeleteSound(inmaze);

}

int main()
{
	int ch=0;

	SetupCallbacks();
	sceKernelUtilsMt19937Init(&ctx, time(NULL));
	scePowerSetClockFrequency(333, 333, 166);

	zx_init();

	while (ch != 2)
	{
		ch=menu();

		switch(ch)
		{
			case 0: game(); break;
			case 1: lveditor(); break;
		}
	}

	zx_quit();

    return 0;
}
